==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/exit.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: exit.c,v 1.10 2004/06/25 12:29:02 lwa Exp $";

#include "config.h"
#include <signal.h>
#include <syslog.h>
#include "lmtp.h"

static int got_signal=0;

void siglog(int s, siginfo_t *info, void *uap) {
  static char *reason;
  static char details[100];
  reason = "";
  details[0] = 0;
  switch(s) {
  case SIGILL:
    switch(info->si_code) {
#ifdef ILL_ILLOPC
    case ILL_ILLOPC: reason = " (Illegal opcode)"; break;
#endif
#ifdef ILL_ILLOPN
    case ILL_ILLOPN: reason = " (Illegal operand)"; break;
#endif
#ifdef ILL_ILLADR
    case ILL_ILLADR: reason = " (Illegal addressing mode)"; break;
#endif
#ifdef ILL_ILLTRP
    case ILL_ILLTRP: reason = " (Illegal trap)"; break;
#endif
#ifdef ILL_PRVOPC
    case ILL_PRVOPC: reason = " (Privileged opcode)"; break;
#endif
#ifdef ILL_PRVREG
    case ILL_PRVREG: reason = " (Privileged register)"; break;
#endif
#ifdef ILL_COPROC
    case ILL_COPROC: reason = " (Coprocessor error)"; break;
#endif
#ifdef ILL_BADSTK
    case ILL_BADSTK: reason = " (Internal stack error)"; break;
#endif
    }
    sprintf(details, "SIGILL at %p", info->si_addr);
    break;
  case SIGFPE:
    switch(info->si_code) {
#ifdef FPE_INTDIV
    case FPE_INTDIV: reason = " (Integer divide by zero)"; break;
#endif
#ifdef FPE_INTOVF
    case FPE_INTOVF: reason = " (Integer overflow)"; break;
#endif
#ifdef FPE_FLTDIV
    case FPE_FLTDIV: reason = " (Floating point divide by zero)"; break;
#endif
#ifdef FPE_FLTOVF
    case FPE_FLTOVF: reason = " (Floating point overflow)"; break;
#endif
#ifdef FPE_FLTUND
    case FPE_FLTUND: reason = " (Floating point underflow)"; break;
#endif
#ifdef FPE_FLTRES
    case FPE_FLTRES: reason = " (Floating poing inexact result)"; break;
#endif
#ifdef FPE_FLTINV
    case FPE_FLTINV: reason = " (Invalid Floating poing operation)"; break;
#endif
#ifdef FPE_FLTSUB
    case FPE_FLTSUB: reason = " (Subscript out of range)"; break;
#endif
    }
    sprintf(details, "SIGFPE at %p", info->si_addr);
    break;
  case SIGSEGV:
    switch(info->si_code) {
#ifdef SEGV_MAPERR
    case SEGV_MAPERR: reason = " (Address not mapped to object)"; break;
#endif
#ifdef SEGV_ACCERR
    case SEGV_ACCERR: reason = " (Invalid permissions for mapped object)"; break;
#endif
    }
    sprintf(details, "SIGSEGV at %p", info->si_addr);
    break;
  case SIGBUS:
    switch(info->si_code) {
#ifdef BUS_ADRALN
    case BUS_ADRALN: reason = " (Invalid address alignment)"; break;
#endif
#ifdef BUS_ADRERR
    case BUS_ADRERR: reason = " (Non-existant physical address)"; break;
#endif
#ifdef BUS_OBJERR
    case BUS_OBJERR: reason = " (Object specific hardware error)"; break;
#endif
    }
    sprintf(details, "SIGBUS at %p", info->si_addr);
    break;
  }
  if (info->si_code<0) {
    switch(info->si_code) {
#ifdef SI_USER
    case SI_USER: reason = " (signal generated via kill)"; break;
#endif
#ifdef SI_TIMER
    case SI_TIMER: reason = " (timer expired)"; break;
#endif
#ifdef SI_ASYNCIO
    case SI_ASYNCIO: reason = " (completion of an asynchronous I/O operation)"; break;
#endif
    }
  }
  syslog(LOG_INFO, "Catch signal %s%s, code=%d, aborting", details, reason, info->si_code);
  return;
}

void sigexit (int s) {
  syslog(LOG_INFO, "Catch signal %d, trying to exit", s);
  got_signal=s;
}

void sigalrm (int s) {
  syslog(LOG_INFO, "Timeout");
  got_signal=s;
}

char *exiting(struct fstate *state) {
  switch(got_signal) {
  case 0:
    return NULL;
  case SIGALRM:
    seterr(state, state->timeout_code, "4.4.2", "Timeout");
    state->timeout_code = 0;
    got_signal=0;
    return state->error;
  }  
  return seterr(state, 421, "4.3.2", "administrative restart");
}

void timeout(struct fstate *state, int t, int code) {
  if (t==0)
    return;
  if (code) {
    alarm(t);
    state->timeout_code=code;
  } else {
    if (state->timeout_code && got_signal!=SIGALRM)
      alarm(0);
  }
}

void setsignals(void) {
  struct  sigaction sa;
  sa.sa_sigaction = siglog;
  sa.sa_flags = SA_SIGINFO | SA_RESETHAND;

  signal(SIGTERM, sigexit);
  signal(SIGHUP, sigexit);
  signal(SIGALRM, sigalrm);

  sigaction(SIGSEGV, &sa, NULL);
  sigaction(SIGILL, &sa, NULL);
  sigaction(SIGBUS, &sa, NULL);
  sigaction(SIGFPE, &sa, NULL);

}

==> lmtpd-0.10.7.tar.gz: lmtpd-0.10.7/exit.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: exit.c,v 1.11 2004/10/26 09:59:20 lwa Exp $";


#include <signal.h>
#include <syslog.h>
#include "lmtp.h"

static int got_signal=0;

void siglog(int s, siginfo_t *info, void *uap) {
  static char *reason;
  static char details[100];
  reason = "";
  details[0] = 0;
  switch(s) {
  case SIGILL:
    switch(info->si_code) {
#ifdef ILL_ILLOPC
    case ILL_ILLOPC: reason = " (Illegal opcode)"; break;
#endif
#ifdef ILL_ILLOPN
    case ILL_ILLOPN: reason = " (Illegal operand)"; break;
#endif
#ifdef ILL_ILLADR
    case ILL_ILLADR: reason = " (Illegal addressing mode)"; break;
#endif
#ifdef ILL_ILLTRP
    case ILL_ILLTRP: reason = " (Illegal trap)"; break;
#endif
#ifdef ILL_PRVOPC
    case ILL_PRVOPC: reason = " (Privileged opcode)"; break;
#endif
#ifdef ILL_PRVREG
    case ILL_PRVREG: reason = " (Privileged register)"; break;
#endif
#ifdef ILL_COPROC
    case ILL_COPROC: reason = " (Coprocessor error)"; break;
#endif
#ifdef ILL_BADSTK
    case ILL_BADSTK: reason = " (Internal stack error)"; break;
#endif
    }
    sprintf(details, "SIGILL at %p", info->si_addr);
    break;
  case SIGFPE:
    switch(info->si_code) {
#ifdef FPE_INTDIV
    case FPE_INTDIV: reason = " (Integer divide by zero)"; break;
#endif
#ifdef FPE_INTOVF
    case FPE_INTOVF: reason = " (Integer overflow)"; break;
#endif
#ifdef FPE_FLTDIV
    case FPE_FLTDIV: reason = " (Floating point divide by zero)"; break;
#endif
#ifdef FPE_FLTOVF
    case FPE_FLTOVF: reason = " (Floating point overflow)"; break;
#endif
#ifdef FPE_FLTUND
    case FPE_FLTUND: reason = " (Floating point underflow)"; break;
#endif
#ifdef FPE_FLTRES
    case FPE_FLTRES: reason = " (Floating poing inexact result)"; break;
#endif
#ifdef FPE_FLTINV
    case FPE_FLTINV: reason = " (Invalid Floating poing operation)"; break;
#endif
#ifdef FPE_FLTSUB
    case FPE_FLTSUB: reason = " (Subscript out of range)"; break;
#endif
    }
    sprintf(details, "SIGFPE at %p", info->si_addr);
    break;
  case SIGSEGV:
    switch(info->si_code) {
#ifdef SEGV_MAPERR
    case SEGV_MAPERR: reason = " (Address not mapped to object)"; break;
#endif
#ifdef SEGV_ACCERR
    case SEGV_ACCERR: reason = " (Invalid permissions for mapped object)"; break;
#endif
    }
    sprintf(details, "SIGSEGV at %p", info->si_addr);
    break;
  case SIGBUS:
    switch(info->si_code) {
#ifdef BUS_ADRALN
    case BUS_ADRALN: reason = " (Invalid address alignment)"; break;
#endif
#ifdef BUS_ADRERR
    case BUS_ADRERR: reason = " (Non-existant physical address)"; break;
#endif
#ifdef BUS_OBJERR
    case BUS_OBJERR: reason = " (Object specific hardware error)"; break;
#endif
    }
    sprintf(details, "SIGBUS at %p", info->si_addr);
    break;
  }
  if (info->si_code<0) {
    switch(info->si_code) {
#ifdef SI_USER
    case SI_USER: reason = " (signal generated via kill)"; break;
#endif
#ifdef SI_TIMER
    case SI_TIMER: reason = " (timer expired)"; break;
#endif
#ifdef SI_ASYNCIO
    case SI_ASYNCIO: reason = " (completion of an asynchronous I/O operation)"; break;
#endif
    }
  }
  syslog(LOG_INFO, "Catch signal %s%s, code=%d, aborting", details, reason, info->si_code);
  return;
}

void sigexit (int s) {
  syslog(LOG_INFO, "Catch signal %d, trying to exit", s);
  got_signal=s;
}

void sigalrm (int s) {
  syslog(LOG_INFO, "Timeout");
  got_signal=s;
}

char *exiting(struct fstate *state) {
  switch(got_signal) {
  case 0:
    return NULL;
  case SIGALRM:
    seterr(state, state->timeout_code, "4.4.2", "Timeout");
    state->timeout_code = 0;
    got_signal=0;
    return state->error;
  }  
  return seterr(state, 421, "4.3.2", "administrative restart");
}

void timeout(struct fstate *state, int t, int code) {
  if (t==0)
    return;
  if (code) {
    alarm(t);
    state->timeout_code=code;
  } else {
    if (state->timeout_code && got_signal!=SIGALRM)
      alarm(0);
  }
}

void setsignals(void) {
  struct  sigaction sa;
  sa.sa_sigaction = siglog;
  sa.sa_flags = SA_SIGINFO | SA_RESETHAND;

  signal(SIGTERM, sigexit);
  signal(SIGHUP, sigexit);
  signal(SIGALRM, sigalrm);

  sigaction(SIGSEGV, &sa, NULL);
  sigaction(SIGILL, &sa, NULL);
  sigaction(SIGBUS, &sa, NULL);
  sigaction(SIGFPE, &sa, NULL);

}
